﻿// This file contains your Data Connector logic
section PetroVisorWebApi;

//OAuth2 settings
redirect_uri = "https://oauth.powerbi.com/views/oauthredirect.html";
client_id = "power.bi"; 
token_uri = "https://identity.us1.petrovisor.com/connect/token";
authorize_uri = "https://identity.us1.petrovisor.com/connect/authorize";
logout_uri = "https://identity.us1.petrovisor.com/connect/endsession";
webapi_uri = "https://api.us1.petrovisor.com";

[DataSource.Kind="PetroVisorWebApi", Publish="PetroVisorWebApi.Publish"]
shared PetroVisor.PetroVisorWebApi = Value.ReplaceType(PetroVisor.PetroVisorWebApi.Impl, WebApiArgsType);

WebApiArgsType = type function (
    Request as (type text meta [
        Documentation.FieldCaption = "Web API GET request",
        Documentation.FieldDescription = "request (see swagger)",
        Documentation.SampleValues = {"Configuration/ServerVersion"}
    ]),
    Workspace as (type text meta [
        Documentation.FieldCaption = "Workspace name",
        Documentation.FieldDescription = "PetroVisor workspace name",
        Documentation.SampleValues = {"Asset IX"}
    ]))
    as any meta [
        Documentation.Name = "PetroVisor.Response",
        Documentation.LongDescription = "Get response data from request"
    ];


PetroVisor.PetroVisorWebApi.Impl = (Request as text,
                           Workspace as text) as any =>
    let
        baseURL = webapi_uri & "/PetroVisor/API",
        workspaceURL = if Workspace = null then baseURL else baseURL & "/" & Workspace,
        token_uri = "AAA",
        response = WebData(workspaceURL & "/" & Request)
    in
        response;


WebData = (request as text) =>
    let
        query= request,
        web = Web.Contents(query, [Timeout= #duration(0, 0, 60,0)] ),
        json = Json.Document(web),
        value = json[value]
    in
        json;

// Data Source Kind description
PetroVisorWebApi = [
    TestConnection = (dataSourcePath) =>
        let
            json = Json.Document(dataSourcePath),
            Request = json[Request],
            Workspace = json[Workspace]
        in
            { "PetroVisor.PetroVisorWebApi", Request, Workspace },
    Authentication = [
        // Key = [],
        // UsernamePassword = [],
        // Windows = [],
        // Implicit = []
        OAuth = [
            StartLogin = StartLogin,
            FinishLogin = FinishLogin,
            Refresh = Refresh,
            Logout = Logout,
            Label = Extension.LoadString("AuthenticationLabel")
        ]
    ],
    Label = Extension.LoadString("DataSourceLabel")
];

//
// OAuth2 flow definition
//

// StartLogin builds a record containing the information needed for the client
// to initiate an OAuth flow. Note for the AAD flow, the display parameter is
// not used.
//
// resourceUrl: Derived from the required arguments to the data source function
//              and is used when the OAuth flow requires a specific resource to 
//              be passed in, or the authorization URL is calculated (i.e. when
//              the tenant name/ID is included in the URL). In this example, we
//              are hardcoding the use of the "common" tenant, as specified by
//              the 'authorize_uri' variable.
// state:       Client state value we pass through to the service.
// display:     Used by certain OAuth services to display information to the
//              user.
//
// Returns a record containing the following fields:
// LoginUri:     The full URI to use to initiate the OAuth flow dialog.
// CallbackUri:  The return_uri value. The client will consider the OAuth
//               flow complete when it receives a redirect to this URI. This
//               generally needs to match the return_uri value that was
//               registered for your application/client. 
// WindowHeight: Suggested OAuth window height (in pixels).
// WindowWidth:  Suggested OAuth window width (in pixels).
// Context:      Optional context value that will be passed in to the FinishLogin
//               function once the redirect_uri is reached. 
StartLogin = (resourceUrl, state, display) =>
    let
        authorizeUrl = authorize_uri & "?" & Uri.BuildQueryString([
            client_id = client_id,  
            redirect_uri = redirect_uri,
            state = state,
            scope = "offline_access petrovisor.api",
            response_type = "code",
            response_mode = "query",
            login = "login"
        ])
    in
        [
            LoginUri = authorizeUrl,
            CallbackUri = redirect_uri,
            WindowHeight = 720,
            WindowWidth = 1024,
            Context = null
        ];

// FinishLogin is called when the OAuth flow reaches the specified redirect_uri. 
// Note for the AAD flow, the context and state parameters are not used. 
//
// context:     The value of the Context field returned by StartLogin. Use this to 
//              pass along information derived during the StartLogin call (such as
//              tenant ID)
// callbackUri: The callbackUri containing the authorization_code from the service.
// state:       State information that was specified during the call to StartLogin. 
FinishLogin = (context, callbackUri, state) =>
    let
        // parse the full callbackUri, and extract the Query string
        parts = Uri.Parts(callbackUri)[Query],
        // if the query string contains an "error" field, raise an error
        // otherwise call TokenMethod to exchange our code for an access_token
        result = if (Record.HasFields(parts, {"error", "error_description"})) then 
                    error Error.Record(parts[error], parts[error_description], parts)
                 else
                    TokenMethod("authorization_code", "code", parts[code])
    in
        result;

// Called when the access_token has expired, and a refresh_token is available.
// 
Refresh = (resourceUrl, refresh_token) => TokenMethod("refresh_token", "refresh_token", refresh_token);

Logout = (token) => logout_uri;


// grantType:  Maps to the "grant_type" query parameter.
// tokenField: The name of the query parameter to pass in the code.
// code:       Is the actual code (authorization_code or refresh_token) to send to the service.
TokenMethod = (grantType, tokenField, code) =>
    let
        queryString = [
            client_id = client_id,
            scope = "offline_access petrovisor.api",
            grant_type = grantType,
            redirect_uri = redirect_uri
        ],
        queryWithCode = Record.AddField(queryString, tokenField, code),

        tokenResponse = Web.Contents(token_uri, [
            Content = Text.ToBinary(Uri.BuildQueryString(queryWithCode)),
            Headers = [
                #"Content-type" = "application/x-www-form-urlencoded",
                #"Accept" = "application/json"
            ],
            ManualStatusHandling = {400} 
        ]),
        body = Json.Document(tokenResponse),
        result = if (Record.HasFields(body, {"error", "error_description"})) then 
                    error Error.Record(body[error], body[error_description], body)
                 else
                    body
    in
        result;


// Data Source UI publishing description
PetroVisorWebApi.Publish = [
    Beta = false,
    Category = "Other",
    ButtonText = { Extension.LoadString("ButtonTitle"), Extension.LoadString("ButtonHelp") },
    LearnMoreUrl = "https://powerbi.microsoft.com/",
    SourceImage = PetroVisorWebApi.Icons,
    SourceTypeImage = PetroVisorWebApi.Icons
];

PetroVisorWebApi.Icons = [
    Icon16 = { Extension.Contents("PetroVisor16.png"), Extension.Contents("PetroVisor20.png"), Extension.Contents("PetroVisor24.png"), Extension.Contents("PetroVisor32.png") },
    Icon32 = { Extension.Contents("PetroVisorWebApi32.png"), Extension.Contents("PetroVisorWebApi40.png"), Extension.Contents("PetroVisorWebApi48.png"), Extension.Contents("PetroVisorWebApi64.png") }
];
